"""Product block for `Opengear` products."""

import ipaddress

from orchestrator.domain.base import ProductBlockModel
from orchestrator.types import SubscriptionLifecycle

from gso.products.product_blocks.site import (
    SiteBlock,
    SiteBlockInactive,
    SiteBlockProvisioning,
)


class OpengearBlockInactive(
    ProductBlockModel,
    lifecycle=[SubscriptionLifecycle.INITIAL],
    product_block_name="OpengearBlock",
):
    """An Opengear that's being currently inactive. See `OpengearBlock`."""

    opengear_hostname: str | None = None
    opengear_site: SiteBlockInactive | None = None
    opengear_wan_address: ipaddress.IPv4Address | None = None
    opengear_wan_netmask: ipaddress.IPv4Address | None = None
    opengear_wan_gateway: ipaddress.IPv4Address | None = None


class OpengearBlockProvisioning(OpengearBlockInactive, lifecycle=[SubscriptionLifecycle.PROVISIONING]):
    """An Opengear that's being provisioned. See `OpengearBlock`."""

    opengear_hostname: str
    opengear_site: SiteBlockProvisioning
    opengear_wan_address: ipaddress.IPv4Address | None
    opengear_wan_netmask: ipaddress.IPv4Address | None
    opengear_wan_gateway: ipaddress.IPv4Address | None


class OpengearBlock(OpengearBlockProvisioning, lifecycle=[SubscriptionLifecycle.ACTIVE]):
    """An Opengear that's currently deployed in the network.

    Attributes:
        opengear_hostname: The hostname of the Opengear device.
        opengear_site: The site where the Opengear device is located.
        opengear_wan_address: The WAN address of the Opengear device.
        opengear_wan_netmask: The WAN netmask of the Opengear device.
        opengear_wan_gateway: The WAN gateway of the Opengear device.
    """

    opengear_hostname: str
    opengear_site: SiteBlock
    opengear_wan_address: ipaddress.IPv4Address
    opengear_wan_netmask: ipaddress.IPv4Address
    opengear_wan_gateway: ipaddress.IPv4Address
