"""Product blocks for VRF Virtual Routing and Forwarding."""

from orchestrator.domain.base import ProductBlockModel
from orchestrator.types import SubscriptionLifecycle

from gso.products.product_blocks.router import RouterBlock, RouterBlockInactive, RouterBlockProvisioning


class VRFBlockInactive(ProductBlockModel, lifecycle=[SubscriptionLifecycle.INITIAL], product_block_name="VRFBlock"):
    """An inactive VRF subscription. See `VRFBlock`."""

    vrf_router_list: list[RouterBlockInactive]
    vrf_name: str | None = None
    route_distinguisher: str | None = None
    route_target: str | None = None
    vrf_as_number: int | None = None


class VRFBlockProvisioning(VRFBlockInactive, lifecycle=[SubscriptionLifecycle.PROVISIONING]):
    """A provisioning VRF subscription. See `VRFBlock`."""

    vrf_router_list: list[RouterBlockProvisioning]  # type: ignore[assignment]
    vrf_name: str
    route_distinguisher: str
    route_target: str
    vrf_as_number: int | None = None


class VRFBlock(VRFBlockProvisioning, lifecycle=[SubscriptionLifecycle.ACTIVE]):
    """Represents an active VRF subscription block.

    Attributes:
        vrf_router_list: List of VRF routers.
        vrf_name: Unique name identifying this VRF.
        route_distinguisher: Route Distinguisher ensuring unique route identification within this VRF.
        route_target: Route Target defining routing policies for importing/exporting routes.
        vrf_as_number: AS number of the VRF.
    """

    vrf_router_list: list[RouterBlock]  # type: ignore[assignment]
    vrf_name: str
    route_distinguisher: str
    route_target: str
    vrf_as_number: int | None = None
