import pytest
from orchestrator.db import db
from orchestrator.domain import SubscriptionModel
from orchestrator.types import SubscriptionLifecycle, UUIDstr

from gso.products import ProductName
from gso.products.product_blocks.site import SiteTier
from gso.products.product_types.site import ImportedSiteInactive, SiteInactive
from gso.services import subscriptions
from gso.utils.types.coordinates import LatitudeCoordinate, LongitudeCoordinate
from gso.utils.types.ip_address import IPAddress
from gso.utils.types.site_name import SiteName


@pytest.fixture()
def site_subscription_factory(faker, geant_partner):
    def subscription_create(
        description: str | None = None,
        site_name: SiteName | None = None,
        site_city: str | None = None,
        site_country: str | None = None,
        site_country_code: str | None = None,
        site_latitude: LatitudeCoordinate | None = None,
        site_longitude: LongitudeCoordinate | None = None,
        site_bgp_community_id: int | None = None,
        site_internal_id: int | None = None,
        site_tier: SiteTier | None = None,
        site_ts_address: IPAddress | None = None,
        status: SubscriptionLifecycle | None = None,
        partner: dict | None = None,
        start_date="2023-05-24T00:00:00+00:00",
        *,
        is_imported: bool | None = True,
        site_contains_optical_equipment: bool | None = True,
    ) -> UUIDstr:
        if partner is None:
            partner = geant_partner

        if is_imported:
            product_id = subscriptions.get_product_id_by_name(ProductName.SITE)
            site_subscription = SiteInactive.from_product_id(product_id, customer_id=partner["partner_id"], insync=True)
        else:
            product_id = subscriptions.get_product_id_by_name(ProductName.IMPORTED_SITE)
            site_subscription = ImportedSiteInactive.from_product_id(
                product_id, customer_id=partner["partner_id"], insync=True
            )

        site_subscription.site.site_city = site_city or faker.city()
        site_subscription.site.site_name = site_name or faker.site_name()
        site_subscription.site.site_country = site_country or faker.country()
        site_subscription.site.site_country_code = site_country_code or faker.country_code()
        site_subscription.site.site_latitude = site_latitude or str(faker.latitude())
        site_subscription.site.site_longitude = site_longitude or str(faker.longitude())
        site_subscription.site.site_bgp_community_id = site_bgp_community_id or faker.pyint()
        site_subscription.site.site_internal_id = site_internal_id or faker.pyint(min_value=1, max_value=254)
        site_subscription.site.site_tier = site_tier or SiteTier.TIER1
        site_subscription.site.site_ts_address = site_ts_address or faker.ipv4()
        site_subscription.site.site_contains_optical_equipment = site_contains_optical_equipment

        site_subscription = SubscriptionModel.from_other_lifecycle(site_subscription, SubscriptionLifecycle.ACTIVE)
        site_subscription.description = description or "Site Subscription"
        site_subscription.start_date = start_date
        if status:
            site_subscription.status = status

        site_subscription.save()
        db.session.commit()

        return str(site_subscription.subscription_id)

    return subscription_create
