import pytest
from orchestrator.db import db
from orchestrator.domain import SubscriptionModel
from orchestrator.types import SubscriptionLifecycle
from pydantic_forms.types import UUIDstr

from gso.products import ProductName
from gso.products.product_blocks.switch import SwitchModel
from gso.products.product_types.site import Site
from gso.products.product_types.switch import ImportedSwitchInactive, SwitchInactive
from gso.services.subscriptions import get_product_id_by_name
from gso.utils.shared_enums import Vendor
from gso.utils.types.ip_address import PortNumber


@pytest.fixture()
def switch_subscription_factory(faker, geant_partner, site_subscription_factory):
    def subscription_create(
        partner: dict | None = None,
        description: str | None = None,
        start_date: str | None = "2024-01-01T10:20:30+01:02",
        fqdn: str | None = None,
        ts_port: PortNumber | None = None,
        site: UUIDstr | None = None,
        switch_vendor: Vendor | None = None,
        switch_model: SwitchModel | None = None,
        status: SubscriptionLifecycle | None = None,
        *,
        is_imported: bool = True,
    ) -> UUIDstr:
        if partner is None:
            partner = geant_partner
        if is_imported:
            product_id = get_product_id_by_name(ProductName.SWITCH)
            switch_subscription = SwitchInactive.from_product_id(product_id, partner["partner_id"])
        else:
            product_id = get_product_id_by_name(ProductName.IMPORTED_SWITCH)
            switch_subscription = ImportedSwitchInactive.from_product_id(product_id, partner["partner_id"])

        switch_subscription.switch.fqdn = fqdn or faker.domain_name(levels=4)
        switch_subscription.switch.ts_port = ts_port or faker.port_number(is_user=True)
        switch_subscription.switch.site = site or Site.from_subscription(site_subscription_factory()).site
        switch_subscription.switch.switch_vendor = switch_vendor or Vendor.JUNIPER
        switch_subscription.switch.switch_model = switch_model or SwitchModel.EX3400_24T

        switch_subscription = SubscriptionModel.from_other_lifecycle(switch_subscription, SubscriptionLifecycle.ACTIVE)
        switch_subscription.insync = True
        switch_subscription.description = description or faker.sentence()
        switch_subscription.start_date = start_date

        if status:
            switch_subscription.status = status

        switch_subscription.save()
        db.session.commit()

        return str(switch_subscription.subscription_id)

    return subscription_create
