import pytest
from orchestrator.types import SubscriptionLifecycle

from gso.products.product_blocks.edge_port import EdgePortType, EncapsulationType
from gso.products.product_types.edge_port import ImportedEdgePort
from gso.utils.shared_enums import Vendor
from gso.utils.types.interfaces import PhysicalPortCapacity
from test.workflows import assert_complete, extract_state, run_workflow


@pytest.fixture()
def imported_edge_port_creation_input_form_data(router_subscription_factory, partner_factory, faker):
    return {
        "node": router_subscription_factory(vendor=Vendor.NOKIA),
        "service_type": EdgePortType.CUSTOMER,
        "speed": PhysicalPortCapacity.TEN_GIGABIT_PER_SECOND,
        "encapsulation": EncapsulationType.DOT1Q,
        "name": "lag34",
        "minimum_links": 2,
        "ga_id": faker.imported_ga_id(),
        "mac_address": faker.mac_address(),
        "partner": partner_factory()["name"],
        "enable_lacp": True,
        "ignore_if_down": False,
        "ae_members": [
            {
                "interface_name": faker.network_interface(),
                "interface_description": faker.sentence(),
            },
            {
                "interface_name": faker.network_interface(),
                "interface_description": faker.sentence(),
            },
        ],
        "description": faker.sentence(),
    }


def test_create_imported_edge_port_success(faker, imported_edge_port_creation_input_form_data):
    result, _, _ = run_workflow("create_imported_edge_port", [imported_edge_port_creation_input_form_data])
    state = extract_state(result)
    subscription = ImportedEdgePort.from_subscription(state["subscription_id"])
    assert_complete(result)
    assert subscription.status == SubscriptionLifecycle.ACTIVE
