import pytest
from orchestrator.types import SubscriptionLifecycle

from gso.products.product_blocks.layer_2_circuit import Layer2CircuitType
from gso.products.product_types.layer_2_circuit import Layer2Circuit, Layer2CircuitServiceType
from gso.utils.helpers import generate_unique_vc_id
from test.workflows import assert_complete, extract_state, run_workflow


@pytest.mark.parametrize(
    "layer_2_service_type", [Layer2CircuitServiceType.GEANT_PLUS, Layer2CircuitServiceType.EXPRESSROUTE]
)
def test_create_imported_layer_2_circuit_success(
    faker, partner_factory, edge_port_subscription_factory, layer_2_service_type
):
    partner = partner_factory()
    edge_port_a = edge_port_subscription_factory(partner=partner)
    edge_port_b = edge_port_subscription_factory(partner=partner)
    policer_enabled = faker.boolean()
    creation_form_input_data = [
        {
            "service_type": layer_2_service_type,
            "partner": partner["name"],
            "layer_2_circuit_type": Layer2CircuitType.TAGGED,
            "policer_enabled": policer_enabled,
            "vlan_range_lower_bound": faker.vlan_id(),
            "vlan_range_upper_bound": faker.vlan_id(),
            "vc_id": generate_unique_vc_id(),
            "policer_bandwidth": faker.bandwidth() if policer_enabled else None,
            "policer_burst_rate": faker.bandwidth() if policer_enabled else None,
            "gs_id": faker.imported_gs_id(),
            "layer_2_circuit_side_a": {"edge_port": edge_port_a, "vlan_id": faker.vlan_id()},
            "layer_2_circuit_side_b": {"edge_port": edge_port_b, "vlan_id": faker.vlan_id()},
        }
    ]

    result, _, _ = run_workflow("create_imported_layer_2_circuit", creation_form_input_data)
    state = extract_state(result)
    assert_complete(result)
    subscription = Layer2Circuit.from_subscription(state["subscription_id"])
    assert subscription.status == SubscriptionLifecycle.ACTIVE
    assert subscription.layer_2_circuit.layer_2_circuit_type == Layer2CircuitType.TAGGED
    assert subscription.layer_2_circuit.virtual_circuit_id == creation_form_input_data[0]["vc_id"]
    assert len(subscription.layer_2_circuit.layer_2_circuit_sides) == 2
    assert subscription.layer_2_circuit.layer_2_circuit_sides[0].sbp.is_tagged is True
    assert subscription.layer_2_circuit.layer_2_circuit_sides[0].sbp.gs_id == creation_form_input_data[0]["gs_id"]
    assert (
        str(subscription.layer_2_circuit.layer_2_circuit_sides[0].sbp.edge_port.owner_subscription_id)
        == creation_form_input_data[0]["layer_2_circuit_side_a"]["edge_port"]
    )
    assert subscription.layer_2_circuit.policer_enabled == policer_enabled
    assert subscription.layer_2_circuit.bandwidth == creation_form_input_data[0]["policer_bandwidth"]
    assert subscription.layer_2_circuit.policer_burst_rate == creation_form_input_data[0]["policer_burst_rate"]
