import pytest
from orchestrator.types import SubscriptionLifecycle

from gso.products.product_blocks.bgp_session import IPFamily
from gso.products.product_types.l3_core_service import ImportedL3CoreService, L3CoreServiceType
from gso.utils.shared_enums import SBPType
from test.workflows import assert_complete, extract_state, run_workflow


@pytest.mark.parametrize(
    "l3_core_service_type",
    [
        L3CoreServiceType.GEANT_IP,
        L3CoreServiceType.IAS,
        L3CoreServiceType.GWS,
        L3CoreServiceType.LHCONE,
        L3CoreServiceType.COPERNICUS,
    ],
)
def test_create_imported_l3_core_service_success(
    faker, partner_factory, edge_port_subscription_factory, l3_core_service_type
):
    creation_form_input_data = {
        "partner": partner_factory()["name"],
        "service_type": l3_core_service_type,
        "service_binding_ports": [
            {
                "edge_port": edge_port_subscription_factory(),
                "ap_type": "PRIMARY",
                "gs_id": faker.imported_gs_id(),
                "sbp_type": SBPType.L3,
                "is_tagged": faker.boolean(),
                "vlan_id": faker.vlan_id(),
                "ipv4_address": faker.ipv4(),
                "ipv4_mask": faker.ipv4_netmask(),
                "ipv6_address": faker.ipv6(),
                "ipv6_mask": faker.ipv6_netmask(),
                "custom_firewall_filters": faker.boolean(),
                "v4_bfd_settings": {
                    "bfd_enabled": True,
                    "bfd_interval_tx": faker.pyint(),
                    "bfd_interval_rx": faker.pyint(),
                    "bfd_multiplier": faker.pyint(),
                },
                "v6_bfd_settings": {
                    "bfd_enabled": True,
                    "bfd_interval_tx": faker.pyint(),
                    "bfd_interval_rx": faker.pyint(),
                    "bfd_multiplier": faker.pyint(),
                },
                "bgp_peers": [
                    {
                        "bfd_enabled": faker.boolean(),
                        "has_custom_policies": faker.boolean(),
                        "authentication_key": faker.password(),
                        "multipath_enabled": faker.boolean(),
                        "send_default_route": faker.boolean(),
                        "is_passive": faker.boolean(),
                        "peer_address": faker.ipv4(),
                        "families": [IPFamily.V4UNICAST, IPFamily.V4MULTICAST],
                        "is_multi_hop": faker.boolean(),
                        "rtbh_enabled": faker.boolean(),
                    },
                    {
                        "bfd_enabled": faker.boolean(),
                        "has_custom_policies": faker.boolean(),
                        "authentication_key": faker.password(),
                        "multipath_enabled": faker.boolean(),
                        "send_default_route": faker.boolean(),
                        "is_passive": faker.boolean(),
                        "peer_address": faker.ipv6(),
                        "families": [IPFamily.V6UNICAST],
                        "is_multi_hop": faker.boolean(),
                        "rtbh_enabled": faker.boolean(),
                    },
                ],
            }
        ],
    }

    result, _, _ = run_workflow("create_imported_l3_core_service", [creation_form_input_data])
    state = extract_state(result)
    subscription = ImportedL3CoreService.from_subscription(state["subscription_id"])
    assert_complete(result)
    assert subscription.status == SubscriptionLifecycle.ACTIVE
