import pytest
from orchestrator.types import SubscriptionLifecycle

from gso.products import ProductName
from gso.products.product_types.lan_switch_interconnect import ImportedLanSwitchInterconnect
from gso.utils.types.virtual_identifiers import DEFAULT_DCN_MANAGEMENT_VLAN_ID, DEFAULT_SWITCH_MANAGEMENT_VLAN_ID
from test.workflows import (
    assert_complete,
    extract_state,
    run_workflow,
)


@pytest.fixture()
def workflow_input_data(faker, router_subscription_factory, switch_subscription_factory):
    return {
        "lan_switch_interconnect_description": faker.sentence(),
        "minimum_links": 1,
        "router_side": {
            "node": router_subscription_factory(),
            "ae_iface": faker.nokia_lag_interface_name(),
            "ae_members": faker.link_members_nokia(),
        },
        "switch_side": {
            "switch": switch_subscription_factory(),
            "ae_iface": faker.juniper_ae_interface_name(),
            "ae_members": faker.link_members_juniper(),
        },
    }


@pytest.mark.workflow()
def test_create_imported_lan_switch_interconnect_success(workflow_input_data):
    result, _, _ = run_workflow("create_imported_lan_switch_interconnect", [workflow_input_data])
    state = extract_state(result)
    subscription = ImportedLanSwitchInterconnect.from_subscription(state["subscription_id"])

    assert_complete(result)
    assert subscription.product.name == ProductName.IMPORTED_LAN_SWITCH_INTERCONNECT
    assert subscription.status == SubscriptionLifecycle.ACTIVE
    assert subscription.lan_switch_interconnect.dcn_management_vlan_id == DEFAULT_DCN_MANAGEMENT_VLAN_ID
    assert subscription.lan_switch_interconnect.switch_management_vlan_id == DEFAULT_SWITCH_MANAGEMENT_VLAN_ID


@pytest.mark.workflow()
def test_create_imported_lan_switch_interconnect_custom_vlan_ids(faker, workflow_input_data):
    custom_switch_vlan_id = faker.vlan_id()
    custom_dcn_vlan_id = faker.vlan_id()
    workflow_input_data.update({
        "switch_management_vlan_id": custom_switch_vlan_id,
        "dcn_management_vlan_id": custom_dcn_vlan_id,
    })
    result, _, _ = run_workflow("create_imported_lan_switch_interconnect", [workflow_input_data])
    state = extract_state(result)
    subscription = ImportedLanSwitchInterconnect.from_subscription(state["subscription_id"])

    assert_complete(result)
    assert subscription.product.name == ProductName.IMPORTED_LAN_SWITCH_INTERCONNECT
    assert subscription.status == SubscriptionLifecycle.ACTIVE
    assert subscription.lan_switch_interconnect.dcn_management_vlan_id == custom_dcn_vlan_id
    assert subscription.lan_switch_interconnect.switch_management_vlan_id == custom_switch_vlan_id
