from unittest.mock import patch

import pytest

from gso.products.product_types.lan_switch_interconnect import LanSwitchInterconnect
from test.workflows import assert_complete, extract_state, run_workflow


@pytest.mark.workflow()
@patch("gso.services.infoblox.find_host_by_fqdn")
@patch("gso.services.infoblox.delete_network")
def test_terminate_lan_switch_interconnect(
    mock_delete_network, mock_find_host, lan_switch_interconnect_subscription_factory, faker
):
    subscription_id = lan_switch_interconnect_subscription_factory()
    initial_lan_switch_interconnect_data = [{"subscription_id": subscription_id}, {"tt_number": faker.tt_number()}]
    result, _, _ = run_workflow("terminate_lan_switch_interconnect", initial_lan_switch_interconnect_data)
    assert_complete(result)

    state = extract_state(result)
    subscription_id = state["subscription_id"]
    subscription = LanSwitchInterconnect.from_subscription(subscription_id)
    assert subscription.status == "terminated"
    assert mock_find_host.call_count == 2
    assert mock_delete_network.call_count == 1
