from unittest.mock import patch

import pytest
from infoblox_client import objects

from gso.products.product_types.lan_switch_interconnect import LanSwitchInterconnect
from test.workflows import assert_complete, assert_lso_success, extract_state, run_workflow


@pytest.mark.workflow()
@patch("gso.services.infoblox.find_host_by_fqdn")
@patch("gso.services.infoblox.find_network_by_cidr")
@patch("gso.services.lso_client._send_request")
def test_validate_lan_switch_interconnect(
    mock_lso_interaction, mock_find_network, mock_find_host, lan_switch_interconnect_subscription_factory, faker
):
    subscription_id = lan_switch_interconnect_subscription_factory()
    mocked_netbox_reply = objects.HostRecord(
        connector=None,
        aliases=[],
        comment=subscription_id,
        ipv4addrs=[
            objects.IPv4(
                ipv4addr=str(faker.ipv4()),
                configure_for_dhcp=False,
                mac="00:00:00:00:00:00",
                ip=str(faker.ipv4()),
                host=faker.domain_name(levels=4),
            ),
        ],
        name=faker.domain_name(levels=4),
    )
    mock_find_host.return_value = mocked_netbox_reply
    mock_find_network.return_value = mocked_netbox_reply
    initial_lan_switch_interconnect_data = [{"subscription_id": subscription_id}]
    result, process_stat, step_log = run_workflow(
        "validate_lan_switch_interconnect", initial_lan_switch_interconnect_data
    )
    result, _ = assert_lso_success(result, process_stat, step_log)
    assert_complete(result)

    state = extract_state(result)
    subscription_id = state["subscription_id"]
    subscription = LanSwitchInterconnect.from_subscription(subscription_id)
    assert subscription.status == "active"
    assert subscription.insync is True
    assert mock_find_host.call_count == 2
    assert mock_find_network.call_count == 1
