from unittest.mock import patch

import pytest

from gso.products import ProductName
from gso.products.product_blocks.switch import SwitchModel
from gso.products.product_types.switch import Switch
from gso.services.subscriptions import get_product_id_by_name
from test import USER_CONFIRM_EMPTY_FORM
from test.services.conftest import MockedSharePointClient
from test.workflows import (
    assert_complete,
    assert_lso_interaction_success,
    assert_suspended,
    extract_state,
    resume_workflow,
    run_workflow,
)


@pytest.mark.workflow()
@patch("gso.services.lso_client._send_request")
@patch("gso.services.infoblox.hostname_available")
@patch("gso.services.sharepoint.SharePointClient")
def test_create_switch_success(
    mock_sharepoint_client, mock_hostname_available, mock_execute_playbook, faker, site_subscription_factory
):
    product_id = get_product_id_by_name(ProductName.SWITCH)
    initial_form_input = [
        {"product": product_id},
        {
            "tt_number": faker.tt_number(),
            "switch_site": site_subscription_factory(),
            "hostname": faker.domain_word(),
            "ts_port": faker.port_number(is_user=True),
            "model": SwitchModel.EX3400_24T,
        },
        {},
    ]
    mock_hostname_available.return_value = True
    mock_sharepoint_client.return_value = MockedSharePointClient
    result, process_stat, step_log = run_workflow("create_switch", initial_form_input)

    # Two LSO interactions
    for _ in range(2):
        result, step_log = assert_lso_interaction_success(result, process_stat, step_log)

    # Two user prompts
    for _ in range(2):
        assert_suspended(result)
        result, step_log = resume_workflow(process_stat, step_log, input_data=USER_CONFIRM_EMPTY_FORM)

    # One LSO interaction
    result, step_log = assert_lso_interaction_success(result, process_stat, step_log)

    # Sharepoint list created
    assert_suspended(result)
    result, step_log = resume_workflow(process_stat, step_log, input_data=USER_CONFIRM_EMPTY_FORM)

    assert_complete(result)

    state = extract_state(result)
    subscription_id = state["subscription_id"]
    subscription = Switch.from_subscription(subscription_id)
    assert subscription.status == "provisioning"
    assert mock_execute_playbook.call_count == 3
